@@ -41,6 +41,7 @@
 		     rmail-summary-exists rmail-select-summary
 		     rmail-update-summary url-retrieve
 		     temp-directory babel-fetch babel-wash
+		     find-coding-systems-for-charsets
 		     ))
       (maybe-bind '(global-face-data
 		    mark-active transient-mark-mode mouse-selection-click-count
@@ -66,7 +67,7 @@
 		babel-translations babel-history))
   (maybe-fbind '(color-instance-rgb-components temp-directory
 		 glyph-width annotation-glyph window-pixel-width glyph-height
-		 window-pixel-height
+		 window-pixel-height map-extents
 		 make-color-instance color-instance-name specifier-instance
 		 device-type device-class get-popup-menu-response event-object
 		 x-defined-colors read-color add-submenu set-font-family
